/*
 * Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT.
 */

import { InkeepAICore } from "../core.js";
import { encodeJSON, encodeSimple } from "../lib/encodings.js";
import * as M from "../lib/matchers.js";
import { safeParse } from "../lib/schemas.js";
import { RequestOptions } from "../lib/sdks.js";
import { extractSecurity, resolveGlobalSecurity } from "../lib/security.js";
import { pathToFunc } from "../lib/url.js";
import * as components from "../models/components/index.js";
import {
  ConnectionError,
  InvalidRequestError,
  RequestAbortedError,
  RequestTimeoutError,
  UnexpectedClientError,
} from "../models/errors/httpclienterrors.js";
import * as errors from "../models/errors/index.js";
import { SDKError } from "../models/errors/sdkerror.js";
import { SDKValidationError } from "../models/errors/sdkvalidationerror.js";
import * as operations from "../models/operations/index.js";
import { Result } from "../types/fp.js";

export enum ContinueAcceptEnum {
  applicationJson = "application/json",
  textEventStream = "text/event-stream",
}

/**
 * Continue Chat Session
 */
export async function chatSessionContinue(
  client: InkeepAICore,
  chatSessionId: string,
  continueChatSessionWithChatResultInput:
    components.ContinueChatSessionWithChatResultInput,
  options?: RequestOptions & { acceptHeaderOverride?: ContinueAcceptEnum },
): Promise<
  Result<
    operations.ContinueResponse,
    | errors.HTTPValidationError
    | SDKError
    | SDKValidationError
    | UnexpectedClientError
    | InvalidRequestError
    | RequestAbortedError
    | RequestTimeoutError
    | ConnectionError
  >
> {
  const input: operations.ContinueRequest = {
    chatSessionId: chatSessionId,
    continueChatSessionWithChatResultInput:
      continueChatSessionWithChatResultInput,
  };

  const parsed = safeParse(
    input,
    (value) => operations.ContinueRequest$outboundSchema.parse(value),
    "Input validation failed",
  );
  if (!parsed.ok) {
    return parsed;
  }
  const payload = parsed.value;
  const body = encodeJSON(
    "body",
    payload.ContinueChatSessionWithChatResultInput,
    { explode: true },
  );

  const pathParams = {
    chat_session_id: encodeSimple("chat_session_id", payload.chat_session_id, {
      explode: false,
      charEncoding: "percent",
    }),
  };

  const path = pathToFunc("/v0/chat_sessions/{chat_session_id}/chat_results")(
    pathParams,
  );

  const headers = new Headers({
    "Content-Type": "application/json",
    Accept: options?.acceptHeaderOverride
      || "application/json;q=1, text/event-stream;q=0",
  });

  const secConfig = await extractSecurity(client._options.apiKey);
  const securityInput = secConfig == null ? {} : { apiKey: secConfig };
  const context = {
    operationID: "continue",
    oAuth2Scopes: [],
    securitySource: client._options.apiKey,
  };
  const requestSecurity = resolveGlobalSecurity(securityInput);

  const requestRes = client._createRequest(context, {
    security: requestSecurity,
    method: "POST",
    path: path,
    headers: headers,
    body: body,
    timeoutMs: options?.timeoutMs || client._options.timeoutMs || -1,
  }, options);
  if (!requestRes.ok) {
    return requestRes;
  }
  const req = requestRes.value;

  const doResult = await client._do(req, {
    context,
    errorCodes: ["422", "4XX", "5XX"],
    retryConfig: options?.retries
      || client._options.retryConfig,
    retryCodes: options?.retryCodes || ["429", "500", "502", "503", "504"],
  });
  if (!doResult.ok) {
    return doResult;
  }
  const response = doResult.value;

  const responseFields = {
    ContentType: response.headers.get("content-type")
      ?? "application/octet-stream",
    StatusCode: response.status,
    RawResponse: response,
    Headers: {},
  };

  const [result] = await M.match<
    operations.ContinueResponse,
    | errors.HTTPValidationError
    | SDKError
    | SDKValidationError
    | UnexpectedClientError
    | InvalidRequestError
    | RequestAbortedError
    | RequestTimeoutError
    | ConnectionError
  >(
    M.json(200, operations.ContinueResponse$inboundSchema, {
      key: "ChatResult",
    }),
    M.sse(200, operations.ContinueResponse$inboundSchema, {
      key: "ChatResultStream",
    }),
    M.jsonErr(422, errors.HTTPValidationError$inboundSchema),
    M.fail(["4XX", "5XX"]),
  )(response, { extraFields: responseFields });
  if (!result.ok) {
    return result;
  }

  return result;
}
